﻿package
{	
	/*************************************************************************
	
	The OPKBEX class serve as the document class for the keyboard ex.  
	application.  The intention of this class and the keyboard ex. application
	is to provide a clear understanding of how to launch and interact with 
	a virtual keyboard component on the OpenFrame.
	
	*************************************************************************/
	
	import flash.system.ApplicationDomain;
	
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import op.utils.OPEvent;
	import op.framework.OPLink;
	import op.framework.OpenFrameApplication;
	
	public class OPKBEX extends OpenFrameApplication
	{
		
		private var kb_lock:*
		
		public function OPKBEX()
		{
			trace("OPKBEX::Constructor");
			super();
			
			//define the listener for the OPTextField on the stage 
			txt.addEventListener(MouseEvent.CLICK, launchKeyboard, false, 0, true);
			txt_ml.addEventListener(MouseEvent.CLICK, launchKeyboard, false, 0, true);
			txt_123.addEventListener(MouseEvent.CLICK, launchKeyboard, false, 0, true);
			txt_auto.addEventListener(MouseEvent.CLICK, launchKeyboard, false, 0, true);
		}
		
		public function launchKeyboard(e:Event)
		{
			/*
				When the keyboard movie has finished loading we need to create an object 
				containing all neccessary information to pass to the initialize() function 
				of the keyboard application.
			*/
			
			trace("OPKBEX::launchKeyboard");
			var vars:Object = new Object;
			kb_lock = e.currentTarget;
			vars.txt = kb_lock.lbl;

			switch(kb_lock)
			{
				case txt: 
					vars.password = true;
				case txt_ml:
					vars.type = "ABC";
					vars.multi = kb_lock.lbl_txt.multiline;
					break;
				case txt_123:
					vars.type = "123"
					vars.maxChars = 10;
					break;
				case txt_auto:
					vars.type = "ABC";
					vars.autoComplete = true;
					break;
			}

			/*
				Add event listeners to handle dispatched keyboard events. These 
				listeners must be added directly to keyboard
			*/

			var OPKeyboard:Class = OPLink.classDefinition("components", "OPKeyboard") as Class;
			var kb:* = new OPKeyboard(vars);

			kb.addEventListener(OPEvent.SAVE, onKeyboardSave, false, 0, true)
			kb.addEventListener(OPEvent.BACK, onKeyboardBack, false, 0, true)
			kb.addEventListener(OPEvent.CHANGE, onKeyboardChange, true, 0, true);

			kb.name = "keyboard";
	
			/*
				It's best practice to add the loader directly to the display stack. This will make it 
				easy to reference the keyboard on the display stack without creating extra
				references in memory.  Additionally I can call Loader.unload() when	I am done with 
				the keyboard allowing the keyboard to know that it is being destroyed.
			*/
			addChild(kb);
		}
		
		public function onKeyboardSave(e:*)
		{
			/* 
				this will return an OpenFrameComponentEvent, but I am using a wildcard to 
				avoid any possible class conflicts if the keyboard has been loaded in a 
				seperate application domain
			*/
			
			trace("OPKBEX::onKeyboardSave");
			
			var args = e.args;
			
			for(var nm:String in args)
				trace("OPKBEX::onKeyboardSave:: args." + nm + " = " + args[nm]);
			
			/*
				In this case the user has pressed the save button.  Here we can
				validate the text if neccessary, use the aquired text ( in this case
				we are assigning it back to the TextField), and if appropriate, remove 
				the keyboard from the display stack and unload the loader.			
			*/
				
			//assign the text to the textfield on the stage
			kb_lock.lbl = args.rtnTxt;
			kb_lock = null;
				
			/*
				There is no validation in in this application so all scenerios will end with
				removing the keyboard.  Since we added the loader to the display stack it is the
				parent of the keyboard that needs to be removed and unloaded. The keyboard will
				be the target of the incoming event.				
			*/

			var kb:* = getChildByName("keyboard");
			removeChild(kb);
			kb = null;
				
		}
		
		public function onKeyboardChange(e:*)
		{
			trace("OPKBEX::onKeyboardChange:: txt = " + e.args);
			
			e.target.updateAutoComplete([ { lbl:"itm 0" },
			{ lbl:"itm 1" },
			{ lbl:"itm 2" } ] );
		}
		
		function onKeyboardBack(e:*)
		{
			/* 
				this will return an OpenFrameComponentEvent, but I am using a wildcard to 
				avoid any possible class conflicts if the keyboard has been loaded in a 
				seperate application domain
			*/
			
			trace("OPKBEX::onKeyboardBack");
			
			/*
				In this case the user has pressed the back button so we simply need
				to remove the keyboard from the display stack and ensure all reference
				in memory to the keyboad have been nullified.  Since both cases will'
				always remove the keyboard in this example nothing needs to be done here
			*/
			
			kb_lock = null;

			var kb = getChildByName("keyboard");
			removeChild(kb);
		}
		
		
//		public override function onApplicationUnload(e:Event)
//		{
//			trace("OPKBEX::onApplicationUnload");
//			
//			/*
//				since the keyboard is a loader that we named keyboard
//				we can see if we should unload it before being released from
//				memory. This is good practice since the keyboard may need to 
//				clean up reference or timer before being destroyed.
//				
//				Note: getChildByName is slower than keeping keeping an
//				extra pointer to the keyboard around as a class member. I am using
//				getChildByName here for the sake of simplicity.  In most situations
//				the difference in performance will not be noticable.  
//			*/
//		}
		
	}
	
}