﻿package com.openpeak.controls
{
	import op.framework.OPLink;
	import com.openpeak.graphics.*;
	import fl.containers.BaseScrollPane;
	import fl.controls.ScrollBar;
	import flash.display.Graphics;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.DropShadowFilter;
	import flash.geom.Point;
	import op.external.OPTrace;
	import com.openpeak.controls.OPGestureConstants;
	
	// styles
	
	/**
	 * If set to true then the component will render a skinned background.  This must be set before setSize method is called.
	 */
	[Style(name="background", type="Boolean")]
	
	/**
	 * Set to a value in com.openpeak.components.OPBaseScrollPaneTheme to set the look of the scrollbar
	 */
	[Style(name="theme", type="String")]
		
	/**
	 * The OPBaseScrollPane serves as a stylized wrapper for the fl.containers.BaseScrollPane class
	 * and is the base class for OPSelectableList.  Similar to the fl.containers.BaseScrollPane class, 
	 * OPBaseScrollPane is not designed for direct use and should be extended. Most common wrapper methods
	 * have been added. This class also contains methods for user navigation with a natural, "physical" feel.
	 */
	public class OPFBBaseScrollPane extends Sprite
	{
		//private const FRICTION:Number = .09;			//Amount of easing for flinging. Lower = slower animating
		//private const DRAG_THRESHOLD:Number = 10;		//Distance in pixels to accept before beginning drag handling
		//private const DRAG_STRENGTH:Number = 9;			//Used to amplify the strength of a fling
		//private const REST_THRESHOLD:Number = 2;		//Distance from destination until fling animation snaps
		//private const POLL_COUNT:Number = 5;			//Amount of points to average when determining fling strength
		
		protected var throwDestination:Point = new Point();;
		
		private var dragSpeed:Point = new Point();
		private var dragPoints:Array = [];
		private var initialScrollPosY:Number = 0;
		private var initialScrollPosX:Number = 0;
		private var redragCounter:Number = 0;
		
		private var dragging:Boolean = false;
		private var throwingY:Boolean = false;
		private var throwingX:Boolean = false;
		private var clickX:Number = -1;
		private var clickY:Number = -1;
		
		protected var _so:BaseScrollPane;
		protected var _background:Graphics;
		protected var _renderBackground:Boolean = true;
		protected var _theme:String;
		
		private var _width:Number = 1;
		private var _height:Number = 1
		
		private var currentDirection:String = "DOWN";
		private var lasty:Number = 0;
		
		private var dragBoost:int = 0;
		
		private var _gestureProfile:String = "";
		
		/**
		 * The OPBaseScroll pane constructor takes two parameters.  A subclass of the fl.containers.BaseScrollPane class and
		 * a String identifer for the theme defined in OPBaseScrollPaneTheme
		 * 
		 * @param	so A subclass of BaseScrollPane
		 * @param	theme A constant string from OPBaseScrollPaneTheme
		 */
		
		public function OPFBBaseScrollPane(so:BaseScrollPane, theme:String, gestureProfile:String = ""):void
		{
			OPTrace.debug("        LEE OPFBBaseScrollPane 0");
			
			//define background shape
			var bg:Shape = new Shape();
			bg.filters = [new DropShadowFilter(5, 60, 0, 0.75, 5, 5, 1, 3, true)]; 
			addChild(bg);
			
			// save the gesture profile
			_gestureProfile = gestureProfile;

			//hold background graphics object for drawing
			_background = bg.graphics;
			
			//save and init the base scroll pane
			_so = so;
			_so.mouseFocusEnabled = false;
			_so.setStyle("embedFonts", true);
			_so.setStyle("skin", new Sprite());
			_so.setStyle("scrollBarWidth", 10 * 800 / 800 );

			_so.setStyle("scrollArrowHeight", 10 * 410 / 410);
			_so.verticalLineScrollSize = 0;	
			_so.verticalScrollBar.mouseEnabled = false;
			_so.verticalScrollBar.mouseChildren = false;
			_so.verticalScrollBar.mouseFocusEnabled = false;
			_so.useBitmapScrolling = true;
			//skin the base scroll pane
			this.theme = theme;
			
			//add the base scroll pane to the stage
			addChild(_so);
			
			//Add listener to handle physics control
			addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
		}
		
		/**
		 * Registers an event listener object with an EventDispatcher object so that the listener receives notification of an event.
		 *  
		 * @param	type
		 * @param	listener
		 * @param	useCapture
		 * @param	priority
		 * @param	useWeakReference
		 */
		override public function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false):void 
		{
			_so.addEventListener(type, listener, useCapture, priority, useWeakReference);
		}
		
		/**
		 * Retrieves a style property that is set in the style lookup chain of the component.
		 * 
		 * @param	name
		 * @return
		 */
		public function getStyle(name:String):*
		{
			return _so.getStyle(name);
		}
		
		/**
		 * Gets or sets the height of the component, in pixels.
		 */
		override public function get height():Number { return _height; }
		
		/**
		 * @private
		 */
		override public function set height(value:Number):void 
		{
			setSize(width, value);
		}
		
		/**
		 * Gets or sets a value that describes how many pixels to scroll horizontally when a scroll arrow is clicked.
		 */
		public function get horizontalLineScrollSize():Number
		{
			return _so.horizontalLineScrollSize;
		}
		
		/**
		 * @private
		 */
		public function set horizontalLineScrollSize(value:Number):void
		{
			_so.horizontalLineScrollSize = value;
		}
		
		/**
		 * Gets or sets the count of pixels by which to move the scroll thumb on the vertical scroll bar when the scroll bar track is pressed.
		 */
		public function get horizontalPageScrollSize():Number
		{
			return _so.horizontalPageScrollSize;
		}

		/**
		 * @private
		 */
		public function set horizontalPageScrollSize(value:Number):void
		{
			_so.horizontalPageScrollSize = value;
		}
		
		/**
		 * Gets or sets a value that indicates the state of the horizontal scroll bar.
		 */
		public function get horizontalScrollPolicy():String
		{
			return _so.horizontalScrollPolicy;
		}
		
		/**
		 * @private
		 */
		public function set horizontalScrollPolicy(value:String):void
		{
			_so.horizontalScrollPolicy = value;
		}
		
		/**
		 * Marks a property as invalid and redraws the component on the next frame unless otherwise specified. 
		 * 
		 * @param	arg1 The property to be invalidated. 
		 * @param	arg2 A Boolean value that indicates whether the component should be redrawn on the next frame. The default value is true
		 */
		public function invalidate(arg1:String, arg2:Boolean):void
		{
			_so.invalidate(arg1, arg2);		
		}
		
		/**
		 * Removes a listener from the EventDispatcher object.
		 * 
		 * @param	type
		 * @param	listener
		 * @param	useCapture
		 */
		override public function removeEventListener(type:String, listener:Function, useCapture:Boolean = false):void 
		{
			_so.removeEventListener(type, listener, useCapture);
		}
		
		/**
		 * Sets the component to the specified width and height.
		 * 
		 * @param	w
		 * @param	h
		 */
		public function setSize(w:Number, h:Number):void
		{

			//save values
			_width = w;
			_height = h;
			
			_so.setSize(w - 1, h);
			_background.clear();
			
			if (_renderBackground)
			{
				
				switch(_theme)
				{
					case OPFBBaseScrollPaneTheme.LIGHT:
						//ShapeRenderer.drawBgList(w, h, _background);
						break;
					case OPFBBaseScrollPaneTheme.DARK:
						break;
					case OPFBBaseScrollPaneTheme.NO_BACKGROUND:   
					break;	
				}
				
			}
			
		}
		
		/**
		 * Sets a style property on this component instance.
		 * 
		 * @param	name
		 * @param	value
		 */
		public function setStyle(name:String, value:*):void
		{
			switch(name)
			{
				case "theme":
					theme = String(value);
					
					//force redraw if possible
					if(width && height)
						setSize(width, height);
						
					break;
				case "background":
					_renderBackground = Boolean(value);
					
					//force redraw if possible
					if(width && height)
						setSize(width, height);
						
					break;
				default:
					_so.setStyle(name, value);
			}
		}	
		
		/**
		 * Gets or sets the width of the component, in pixels.
		 */
		override public function get width():Number { return _width; }
		
		/**
		 * @private
		 */
		override public function set width(value:Number):void 
		{
			setSize(value, height);
		}
		
		/**
		 * Gets or sets the theme of the component.  String identifiers should match the constants
		 * in OPBaseScrollPaneTheme
		 */
		protected function get theme() : String { return _theme; }
		
		/**
		 * @Private
		 */
		protected function set theme( id:String ):void
		{
		
			switch(id)
			{
				case OPFBBaseScrollPaneTheme.LIGHT:
				
					_so.setStyle("trackUpSkin",  SCROLLINDICATOR_TRACK );
					_so.setStyle("trackDownSkin", SCROLLINDICATOR_TRACK );
					_so.setStyle("trackDisabledSkin", SCROLLINDICATOR_TRACK);
					_so.setStyle("trackOverSkin", SCROLLINDICATOR_TRACK );
			
					_so.setStyle("thumbUpSkin",  SCROLLINDICATOR_THUMB );
					_so.setStyle("thumbDownSkin", SCROLLINDICATOR_THUMB );
					_so.setStyle("thumbDisabledSkin", SCROLLINDICATOR_THUMB );
					_so.setStyle("thumbOverSkin", SCROLLINDICATOR_THUMB );
					_so.setStyle("thumbIcon", SCROLLINDICATOR_THUMB_ICON );
			
					_so.setStyle("upArrowUpSkin", SCROLLINDICATOR_ARROW_UP );
					_so.setStyle("upArrowOverSkin", SCROLLINDICATOR_ARROW_UP );
					_so.setStyle("upArrowDownSkin", SCROLLINDICATOR_ARROW_UP_H );
					_so.setStyle("upArrowDisabledSkin", SCROLLINDICATOR_ARROW_UP_H );
			
					_so.setStyle("downArrowUpSkin", SCROLLINDICATOR_ARROW_DOWN );
					_so.setStyle("downArrowOverSkin", SCROLLINDICATOR_ARROW_DOWN );
					_so.setStyle("downArrowDownSkin", SCROLLINDICATOR_ARROW_DOWN_H );
				
					break;
				case OPFBBaseScrollPaneTheme.DARK:
					
					_so.setStyle("trackUpSkin",  SCROLLBAR_TRACK_BLACK );
					_so.setStyle("trackDownSkin", SCROLLBAR_TRACK_BLACK );
					_so.setStyle("trackDisabledSkin", SCROLLBAR_TRACK_BLACK );
					_so.setStyle("trackOverSkin", SCROLLBAR_TRACK_BLACK );
			
					_so.setStyle("thumbUpSkin",  SCROLLBAR_THUMB_BLACK );
					_so.setStyle("thumbDownSkin", SCROLLBAR_THUMB_BLACK );
					_so.setStyle("thumbDisabledSkin", SCROLLBAR_THUMB_BLACK );
					_so.setStyle("thumbOverSkin", SCROLLBAR_THUMB_BLACK );
					_so.setStyle("thumbIcon", SCROLLBAR_THUMB_ICON_BLACK );
			
					_so.setStyle("upArrowUpSkin", SCROLLBAR_ARROW_UP_BLACK );
					_so.setStyle("upArrowOverSkin", SCROLLBAR_ARROW_UP_BLACK );
					_so.setStyle("upArrowDownSkin", SCROLLBAR_ARROW_UP_H_BLACK );
					_so.setStyle("upArrowDisabledSkin", SCROLLBAR_ARROW_UP_H_BLACK );
			
					_so.setStyle("downArrowUpSkin", SCROLLBAR_ARROW_DOWN_BLACK );
					_so.setStyle("downArrowOverSkin", SCROLLBAR_ARROW_DOWN_BLACK );
					_so.setStyle("downArrowDownSkin", SCROLLBAR_ARROW_DOWN_H_BLACK );
				
					break;					
			}
			
			_theme = id;
			
		}
		
		/**
		 * Gets the vertical ScrollBar
		 * @return scrollBar scrollBar
		 */
		public function get VerticalScrollBar():ScrollBar {
			OPTrace.debug("LEE RETURNING A VERTICAL SCROLLBAR");
			return _so.verticalScrollBar;
		}
		
		/**
		 * Gets or sets a value that describes how many pixels to scroll vertically when a scroll arrow is clicked.
		 */
		public function get verticalLineScrollSize():Number
		{
			return _so.verticalLineScrollSize;
		}
		
		/**
		 * @private
		 */
		public function set verticalLineScrollSize(value:Number):void
		{
			_so.verticalLineScrollSize = value;
		}
		
		/**
		 * Gets or sets the count of pixels by which to move the scroll thumb on the vertical scroll bar when the scroll bar track is pressed.
		 */
		public function get verticalPageScrollSize():Number
		{
			return _so.verticalPageScrollSize;
		}
		
		/**
		 * @private
		 */
		public function set verticalPageScrollSize(value:Number):void
		{
			_so.verticalPageScrollSize = value;
		}
		
		/**
		 * Gets or sets a value that indicates the state of the vertical scroll bar.
		 */
		public function get verticalScrollPolicy():String
		{
			return _so.verticalScrollPolicy
		}
		
		/**
		 * @private
		 */
		public function set verticalScrollPolicy(value:String):void
		{
			_so.verticalScrollPolicy = value;
		}
				
		public function get useBitmapScrolling():Boolean
		{
			return _so.useBitmapScrolling 
		}
		
		public function set useBitmapScrolling(arg:Boolean):void
		{
			_so.useBitmapScrolling = arg;
		}
		////
		//Begin Physics code
		///
		
		/**
		 * Callback for mousedowns within the pane. Set in the constructor
		 * @param	e
		 */
		protected function onMouseDown(e:MouseEvent):void
		{
			
			//Begin tracking the cursor location to track the drag speed
			dragPoints = [getMousePoint()];
			clickY = mouseY;
			clickX = mouseX;
			
			addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);

			if (throwing)
			{
				removeEventListener(Event.ENTER_FRAME, onThrowFrame);
				initialScrollPosY = _so.verticalScrollPosition;
				initialScrollPosX = _so.horizontalScrollPosition;
				dragging = true;	
				_so.mouseChildren = false;
			}	
			
			addEventListener(Event.ENTER_FRAME, onDragFrame);
		}
	
		/**
		 * Callback on enterFrame when the pane is being dragged. Set in onMouseDown.
		 * @param	e
		 */
		private function onDragFrame(e:Event):void
		{
			if(dragPoints.length >= OPGestureConstants.getCustomValue(_gestureProfile,"POLL_COUNT")-1)
				dragPoints.shift();

			if(!dragging)
			{	
				if ( Math.abs(mouseY - clickY) > OPGestureConstants.getCustomValue(_gestureProfile,"DRAG_THRESHOLD"))
				{
					initialScrollPosX = _so.horizontalScrollPosition;
					initialScrollPosY = _so.verticalScrollPosition;
					dragging = true;
				}
				if ( Math.abs(mouseX - clickX) > OPGestureConstants.getCustomValue(_gestureProfile,"DRAG_THRESHOLD"))
				{
					initialScrollPosX = _so.horizontalScrollPosition;
					initialScrollPosY = _so.verticalScrollPosition;
					dragging = true;
				}
			}
			else
			{
				//Prevent selection of any list items
				_so.mouseChildren = false;
				
				if (_so.horizontalScrollPosition != initialScrollPosX + (clickX - mouseX))
					_so.horizontalScrollPosition = initialScrollPosX + (clickX - mouseX);
				
				if (_so.verticalScrollPosition != initialScrollPosY + (clickY - mouseY))
					_so.verticalScrollPosition = initialScrollPosY + (clickY - mouseY);
					
				dragPoints.push(getMousePoint());
			}	
		}
		
		/**
		 * Callback on mouseUp. Set in onMouseDown
		 * If the pane is being dragged => Fling the pane.
		 * @param	e
		 */
		protected function onMouseUp(e:Event):void
		{
			if(dragging)
			{
				dragging = false;
				var lastDragSpeed:Point = dragSpeed;
				dragSpeed = new Point();
				dragPoints.push(getMousePoint());
				
				//Shouldn't be a problem, but let's avoid an divbyzero errors
				if(dragPoints.length - 1 == 0)
					dragPoints = [getMousePoint(), getMousePoint()]
				
				//start averaging the sampled points to determine fling speed
				for (var n:Number = 1; n < dragPoints.length; n++ )
				{
					dragSpeed = dragSpeed.add(dragPoints[n - 1].subtract(dragPoints[n]));
				}	
				

				dragSpeed.x = dragSpeed.x / (dragPoints.length - 1);
				dragSpeed.y = ( dragSpeed.y / (dragPoints.length - 1) );	
					
				//If we're already flinging, and the new fling is in the same direction => Average the two speeds
				if (throwing && dragPoints.length < OPGestureConstants.getCustomValue(_gestureProfile,"POLL_COUNT")) //&& ((lastDragSpeed > 0) == (dragSpeed > 0))
				{	
					if(Math.abs(dragSpeed.x) < OPGestureConstants.getCustomValue(_gestureProfile,"REST_THRESHOLD"))
						dragSpeed.x = 0;
					if(Math.abs(dragSpeed.y) < OPGestureConstants.getCustomValue(_gestureProfile,"REST_THRESHOLD"))
						dragSpeed.y = 0;
					
					//else
					//	dragSpeed = (dragSpeed + lastDragSpeed)/2;
					
					// Boost the Drag Strenght for consective swipes
					if (currentDirection == "DOWN") {
						dragBoost+=OPGestureConstants.getCustomValue(_gestureProfile,"DRAG_BOOST");
					}else {
						dragBoost-=OPGestureConstants.getCustomValue(_gestureProfile,"DRAG_BOOST");
					}
					
					
		
				}

				throwDestination = new Point(( OPGestureConstants.getCustomValue(_gestureProfile,"DRAG_STRENGTH") * dragSpeed.x + _so.horizontalScrollPosition),
											( ( OPGestureConstants.getCustomValue(_gestureProfile,"DRAG_STRENGTH") + dragBoost ) * dragSpeed.y + _so.verticalScrollPosition));
				
				/*  Ease into the extremes
				if (throwDestination > _so.maxVerticalScrollPosition)
					throwDestination = _so.maxVerticalScrollPosition;
				else if (throwDestination < 0)
					throwDestination = 0;
				*/
				
				addEventListener(Event.ENTER_FRAME, onThrowFrame);
				removeEventListener(Event.ENTER_FRAME, onDragFrame);
				
				throwingX = true;
				throwingY = true;
			}
			removeEventListener(Event.ENTER_FRAME, onDragFrame);
			removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}
		
		/**
		 * Callback on enterFrame when the pane is being thrown. Set in onMouseUp.
		 * @param	e
		 */
		protected function onThrowFrame(e:Event):void
		{
			
			
			//Ease the vertical position toward its destination
			_so.horizontalScrollPosition +=  (throwDestination.x - _so.horizontalScrollPosition) * OPGestureConstants.getCustomValue(_gestureProfile,"FRICTION"); 			
			_so.verticalScrollPosition +=   (throwDestination.y - _so.verticalScrollPosition) * OPGestureConstants.getCustomValue(_gestureProfile,"FRICTION"); 
			
			// Determine which direction we are travelling in.
			// This is needed by the dragBoost property
			if (lasty < throwDestination.y) {
				currentDirection = "UP";
			}else {
				currentDirection = "DOWN";
			}
			lasty = throwDestination.y;
					
			//If the vertical position has arrived at its destination, or is at the bottom or the top of the pane => Stop the pane from moving
			if ( Math.abs(throwDestination.y - _so.verticalScrollPosition) < OPGestureConstants.getCustomValue(_gestureProfile,"REST_THRESHOLD") || 
				(throwDestination.y > _so.maxVerticalScrollPosition && _so.verticalScrollPosition == _so.maxVerticalScrollPosition) ||
				(throwDestination.y < 0 && _so.verticalScrollPosition == 0))
			{
				killThrow(false,true);
			}
			if ( Math.abs(throwDestination.x - _so.horizontalScrollPosition) < OPGestureConstants.getCustomValue(_gestureProfile,"REST_THRESHOLD") || 
				(throwDestination.x > _so.maxHorizontalScrollPosition && _so.horizontalScrollPosition == _so.maxHorizontalScrollPosition) ||
				(throwDestination.x < 0 && _so.horizontalScrollPosition == 0))
			{
				killThrow(true,false);
			}
		}
		
		/**
		 * Halts any throw action currently happening within the OPBaseScrollPane 
		 */
		
		public function killThrow(killX:Boolean = true, killY:Boolean = true ):void
		{
			if (throwing)
			{	
				if (killX)
				{
					throwingX = false;
					_so.horizontalScrollPosition = throwDestination.x;
				}
				if (killY)
				{
					throwingY = false;
					_so.verticalScrollPosition = throwDestination.y;
					dragBoost = 0;
				}
				if (!throwing)
				{
					removeEventListener(Event.ENTER_FRAME, onThrowFrame);
					_so.mouseChildren = true;
				}
			}
		}
		public function getMousePoint():Point
		{
			return new Point(mouseX, mouseY);
		}
		public function get throwing():Boolean
		{
			return (throwingY || throwingX);
		}
	}
	
}