﻿package op.extensions.phone
{
	import op.external.*;
	import op.extensions.phone.events.*;
	import flash.errors.IllegalOperationError;
	import flash.utils.Timer;
	import flash.events.TimerEvent;

	/*
	 * This class interfaces with the class module libCallControlAS3.so
	 * There should only be a singleton instance of this through libs/libCallControl.swf
	 */

	public class CallControlFrameClient extends OPClassModuleInstance implements ICallControl
	{
		private static var m_type:Number = libCallControl.CC_RESIDENTIAL;
		private static var _instance = null;
		private var m_accountArray:Array = new Array();
		private var m_activeLine:* = null;
		private var m_initialized:Boolean = false;
		private var click2dialLaunch:Boolean = false;
		private var m_rangeStatResult:String = "";
		private var m_timer:Timer = null;
		
		public function CallControlFrameClient()
		{
			OPTrace.informational("CallControlFrameClient::Constructor");
			if ( _instance == null ) {
				
				// each time super is called, this creates an instance of callcontrol class module (more than one duplicates events)
				// therefore ensure super is only called once
				super("CallControlAS3");
				_instance = this;

				try {
					//manually startup callcontrol class module and then mark it initialized
					if( !startup() ) {
						initialize();

						//Now that we are initialized, get the subscription/range level indication
						getRangeStat();
					}
					else
						OPTrace.severe("CallControlFrameClient::Constructor: ERROR - Unable to initialize Frame Client.");
				}
				catch (e:*) {
					OPTrace.severe("CallControlFrameClient::Constructor: ERROR - Unable to initialize/utilize Frame Client.");
				}
			}
			else
				throw new IllegalOperationError("Cannot instantiate more than ONE CallControlFrameClient");
			
		}

		public function getRangeStat(e:* = null):void
		{
			OPTrace.informational("CallControlFrameClient::getRangeStat");
			if (m_timer != null) {
				m_timer.stop();
				m_timer.removeEventListener(TimerEvent.TIMER, getRangeStat);
				m_timer = null;
			}

			if (m_rangeStatResult == "") {
				var r:int = pt_range_stat();
				if (r) {
					//Anything but 0 means the call failed, so set a timer to retry soon
					m_timer = new Timer(700, 1);
					m_timer.addEventListener(TimerEvent.TIMER, getRangeStat, false, 0, true);
					m_timer.start();
				}
				else {
					//The call was successful. It could take up to 3 seconds to get the event back
					m_timer = new Timer(3000, 1);
					m_timer.addEventListener(TimerEvent.TIMER, getRangeStat, false, 0, true);
					m_timer.start();
				}
			}
		}

		public static function getInstance(t:Number = 0):ICallControl
		{
			return _instance;
		}
		
		public function startup()
		{
			var callPath:String = _className + ".initialize." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function createAccountAndLines():void 
		{
			OPTrace.debug("CallControlFrameClient::createAccountAndLines");

			var line:*;
			var account:Account = new Account();

			account.type = "residential";
			account.provider = "pots";
			account.user = "";
			account.numberOfLines = "2";
			m_accountArray.push(account);

			OPTrace.debug("CallControlFrameClient::Number of lines: " + m_accountArray[0].numberOfLines);

			for (var k:Number = 1; k <= m_accountArray[0].numberOfLines; k++)
			{
				if(m_accountArray[0].type == "enterprise")
					m_type = libCallControl.CC_ENTERPRISE;
				else
					m_type = libCallControl.CC_RESIDENTIAL;

				if ( m_type == libCallControl.CC_ENTERPRISE )
					line = new EnterpriseLine(getInstance(m_type), m_accountArray[0].provider, m_accountArray[0], k);
				else
					line = new ResidentialLine(getInstance(m_type), m_accountArray[0].provider, m_accountArray[0], k);
				m_accountArray[0].lines.push(line);
			}
			m_accountArray[0].featureManager = new FeatureManager(_instance, m_accountArray[0].lines);

			if( m_accountArray.length )
				m_activeLine = m_accountArray[0].lines[0];
		}

		/*
		 * The following functions implement the ICallControl interface.
		 * 
		 ******************** INTERFACE BEGIN ***************************
		 * 
		 */
		public function initialize()
		{
			m_initialized = true;

			createAccountAndLines();
		}

		public function setDefaultLine(line:CallControlLine)
		{
			OPTrace.informational("CallControlFrameClient::setDefaultLine stub");
		}
		
		public function offhook(line:CallControlLine)
		{
			var callPath:String = _className + ".pt_offhook." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function onhook(line:CallControlLine)
		{
			var callPath:String = _className + ".pt_onhook." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function digit(line:CallControlLine, digit:String)
		{
			var callPath:String = _className + ".pt_digit." + _uniqueId;
			var result = OPExternalInterface.call(callPath, digit);
			return result;
		}

		public function dial(line:CallControlLine, dialString:String)
		{
			var callPath:String = _className + ".pt_dial." + _uniqueId;
			var result = OPExternalInterface.call(callPath, dialString);
			return result;
		}

		public function redial(line:CallControlLine)
		{
			var callPath:String = _className + ".redial." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function click2dial(dialString:String, launchGUI:Boolean = true)
		{
			if( m_type != libCallControl.CC_RESIDENTIAL || !activeLine.offHookCounter ) {
				click2dialLaunch = launchGUI;
				activeLine.setTargetNumber(dialString);
				var callPath:String = _className + ".pt_dial." + _uniqueId;
				var result = OPExternalInterface.call(callPath, dialString);
				return result;
			}
		}
		
		public function ignore(line:CallControlLine)
		{
			var callPath:String = _className + ".ignore." + _uniqueId;
			var result = OPExternalInterface.call(callPath, line.getProvider(), line.getAccount(), line.getLineIndex());
			return result;
		}
		
		public function hold(line:CallControlLine)
		{
			var callPath:String = _className + ".hold." + _uniqueId;
			var result = OPExternalInterface.call(callPath, line.getProvider(), line.getAccount(), line.getLineIndex());
			return result;
		}
		
		public function resume(line:CallControlLine)
		{
			var callPath:String = _className + ".resume." + _uniqueId;
			var result = OPExternalInterface.call(callPath, line.getProvider(), line.getAccount(), line.getLineIndex());
			return result;
		}

		public function atransfer(line:CallControlLine, targetLine:Number)
		{
			var callPath:String = _className + ".atransfer." + _uniqueId;
			var result = OPExternalInterface.call(callPath, line.getProvider(), line.getAccount(), line.getLineIndex(), targetLine);
			return result;
		}
		
		public function utransfer(line:CallControlLine, target:String)
		{
			var callPath:String = _className + ".utransfer." + _uniqueId;
			var result = OPExternalInterface.call(callPath, line.getProvider(), line.getAccount(), line.getLineIndex(), target);
			return result;
		}
		
		public function conference(line1:CallControlLine, line2:CallControlLine)
		{
			var callPath:String = _className + ".conference." + _uniqueId;
			var result = OPExternalInterface.call(callPath, line1.getProvider(), line1.getAccount(), line1.getLineIndex(), line2.getProvider(), line2.getAccount(), line2.getLineIndex());
			return result;
		}
		
		public function message(line:CallControlLine, msg:String)
		{
			var callPath:String = _className + ".message." + _uniqueId;
			var result = OPExternalInterface.call(callPath, line.getProvider(), line.getAccount(), line.getLineIndex(), msg);
			return result;
		}

		public function volume(level:Number)
		{
			var callPath:String = _className + ".volume." + _uniqueId;
			var result = OPExternalInterface.call(callPath, level);
			return result;
		}

		public function mute(select:String)
		{
			var callPath:String = _className + ".pt_toggle_mute." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}
		
		public function pageHandsets()
		{
			OPTrace.informational("CallControlFrameClient: page handsets");
			var callPath:String = _className + ".pageHandsets." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}
		
		public function pageCancel()
		{
			var callPath:String = _className + ".pageCancel." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}
		
		public function startRegistration()
		{
			var callPath:String = _className + ".startRegistration." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function deleteHandset(id:String)
		{
			var callPath:String = _className + ".deleteHandset." + _uniqueId;
			var result = OPExternalInterface.call(callPath, id);
			return result;
		}

		public function getHandsetExtendedStatus(id:String)
		{
			var callPath:String = _className + ".getHandsetExtendedStatus." + _uniqueId;
			var result = OPExternalInterface.call(callPath, id);
			return result;
		}
		
		public function getRegisteredHandsets()
		{
			var callPath:String = _className + ".getRegisteredHandsets." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}
		
		public function intercom(type:String, id:String)
		{
			var callPath:String = _className + ".intercom." + _uniqueId;
			var result = OPExternalInterface.call(callPath, type, id);
			return result;
		}
		
		public function setHandsetDateTime(id:String, day:String, month:String, mday:String, year:String, hour:String, min:String)
		{
			var callPath:String = _className + ".setHandsetDateTime." + _uniqueId;
			var result = OPExternalInterface.call(callPath, id, day, month, mday, year, hour, min);
			return result;
		}
		
		public function getAudioEngineStatus()
		{
			var callPath:String = _className + ".getAudioEngineStatus." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}
		
		public function getAccessCode()
		{
			var callPath:String = _className + ".getAccessCode." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}
		
		public function setHandsetName(id:String, name:String)
		{
			var callPath:String = _className + ".setHandsetName." + _uniqueId;
			var result = OPExternalInterface.call(callPath, id, name);
			return result;
		}

		public function setHandsetAddressBookEntry(id:String, ring:String, number:String, name:String)
		{
			var callPath:String = _className + ".setHandsetAddressBookEntry." + _uniqueId;
			var result = OPExternalInterface.call(callPath, id, ring, number, name);
			return result;
		}

		public function resetHandsetAddressBook(id:String)
		{
			var callPath:String = _className + ".resetHandsetAddressBook." + _uniqueId;
			var result = OPExternalInterface.call(callPath, id);
			return result;
		}

		public function get accountArray():Array { return m_accountArray; }
		
		public function set activeLine(a:*):void { m_activeLine = a; }
		
		public function get activeLine():* { return m_activeLine; }
		
		public function get isInitialized():* { return m_initialized; }
		
		public function get launchc2d() : Boolean { return click2dialLaunch; }
		
		public function c2dreset() { click2dialLaunch = false; }

		public function destructor():void
		{
			super.destroy();
		}

		/*
		 * 
		 ******************** INTERFACE END ***************************
		 * 
		 */

		public function get rangestat() : String { return m_rangeStatResult; }

		//Misnamed, but reused for Frame Client to determine subscribed and in-range
		public function get networkisup() : Boolean { return (m_rangeStatResult == "RANGE_IN"); }

		public function get isFrameClient() : Boolean { return true; }

		public function phoneInUse():Boolean
		{
			var inUse:Boolean = false;
			if ( m_activeLine.inBaseIntercom ||  m_activeLine.inHSIntercom != null )
				inUse = true;
			for (var j:Number=0; j < m_accountArray.length && !inUse; j++) {
				for (var k:Number = 1; k <= m_accountArray[j].numberOfLines && !inUse; k++) {
					switch (m_accountArray[j].lines[k - 1].getState()) {
						case Line.STATE_HOLD:
						case Line.STATE_CONNECTED:
						case Line.STATE_REMOTE_ALERTING:
						case Line.STATE_PROCEEDING:
						case Line.STATE_OFF_HOOK:
						case Line.STATE_INCOMING_CALL:
							inUse = true;
						break;
					}
				}
			}
			return inUse;
		}

		public function swap_line()
		{
			if ( m_activeLine.getLineIndex() == 1 ) {
				m_activeLine = m_accountArray[0].lines[1];
			}
			else {
				m_activeLine = m_accountArray[0].lines[0];
			}
		}

		public function enableDND()
		{
			var callPath:String = _className + ".enableDND." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}
		
		public function disableDND()
		{
			var callPath:String = _className + ".disableDND." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}
		
		public function pt_subscription(accessCode:String)
		{
			var callPath:String = _className + ".pt_subscription." + _uniqueId;
			var result = OPExternalInterface.call(callPath, accessCode);
			return result;
		}
		
		public function pt_range_stat()
		{
			var callPath:String = _className + ".pt_range_stat." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function getFirmwareVersion()
		{
			var callPath:String = _className + ".getFirmwareVersion." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function getVersion()
		{
			var callPath:String = _className + ".getVersion." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function getDectVariant()
		{
			var callPath:String = _className + ".getDectVariant." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function getCatiqVersion()
		{
			var callPath:String = _className + ".getCatiqVersion." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function getLastNumber():String
		{
			var callPath:String = _className + ".pt_get_lnr." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function hookFlash(line:CallControlLine, duration:Number)
		{
			OPTrace.debug("CCAS3::HOOKFLASH");
			var callPath:String = _className + ".pt_hook_flash." + _uniqueId;
			var result = OPExternalInterface.call(callPath);
			return result;
		}

		public function onCallControlEvent(xml:String)
		{
			OPTrace.informational("CallControlFrameClient::onCallControlEvent received " + xml);

			if (m_accountArray.length > 0) {
			try 
			{
				var event:* = CallControlParser.parse(xml);

				/*
				 * Because the FC cannot actually ignore a call, the phone could be ringing an incoming call without
				 *   the pop-up window.  That being said, we need to stop the ringtone when we get a hook or
				 *   disconnected event.
				 */
				if (isFrameClient && (event is HookEvent || event is DisconnectedEvent) )
					m_accountArray[0].stopRingback();

				 if(event.getEventType() == CallControlEvent.CC_EVENT_REGISTRATION)
				{
					OPTrace.debug("CallControlEvent.CC_EVENT_REGISTRATION");
				}
				else if(event.getEventType() == CallControlEvent.CC_EVENT_INFORMATIONAL)
				{
					var informationalEvent=InformationalEvent(event);
					if(informationalEvent.getInformation() == "updated")
					{
						OPTrace.debug("CallControlEvent.CC_EVENT_INFORMATIONAL");
						initialize();
					}
				}
				else if(event.getEventType() == CallControlEvent.CC_EVENT_FRAME_CLIENT)
				{
					OPTrace.debug("CallControlEvent.CC_EVENT_FRAME_CLIENT");
					var frameClientEvent:*		= FrameClientEvent(event);
					var frameEvent:String		= frameClientEvent.getEvent();
					//If this event is a range indication
					if ( frameEvent == "RANGE_UNSUB" || frameEvent == "RANGE_OUT" || frameEvent == "RANGE_IN" )
					{
						var oldRangStatRes:String	= m_rangeStatResult;
						m_rangeStatResult			= frameEvent;

						if ( m_rangeStatResult == "RANGE_IN" ) {
							//We are subscribed and in range, get the range level
							var l:String = frameClientEvent.getLevel();
						}

						//If the result changed, send an info message out to update status
						if ( oldRangStatRes != m_rangeStatResult )
							dispatchEvent(new InformationalEvent(""));
					}
				}
				else {
					if( m_accountArray.length ) {
						if (event is HookEvent ||
							event is IncomingCallEvent || //used primarily for call waiting since beep is immediate
							event is DectCallerIDEvent || //used for first incoming call
							event is DisconnectedEvent ||
							event is ProceedingEvent ||
							event is RemoteAlertingEvent ||
							event is IgnoreCallEvent ||
							event is ConnectedEvent ||
							event is DialPlanErrorEvent ||
							event is CallControlErrorEvent ||
							event is MessageEvent ||
							event is HookFlashEvent ||
							event is HookFlashLongEvent ) {

							if ( m_type == libCallControl.CC_RESIDENTIAL ) {
								//m_accountArray[0].lines[event.getLineIndex() - 1].updateState(event);
								m_accountArray[0].featureManager.updateState(event);

								//We won't get a connected event in the frame client, so artifically create one
								if (event is ProceedingEvent) {
									dispatchEvent(event);
									event = new ConnectedEvent();
									event.setLineIndex(1);
									m_accountArray[0].featureManager.updateState(event);
								}
							}
							else
								m_activeLine.updateState(event);
						}
					}
				}
				dispatchEvent(event);
			}
			catch(e:Error)
			{
				OPTrace.informational("Error parsing event message.");
				OPTrace.debug("msg: " + e.message);
			}
			}
			else {
				OPTrace.severe("ERROR: Unable to create account and lines");
				createAccountAndLines();
			}
		}
		
	}
}
