﻿package op.extensions.phone
{
	import op.external.*;
	import op.extensions.phone.events.*;
	import op.external.OPTrace;

	public class CallControlParser
	{
		private static var iAmFrameClient:Boolean = false;
		private static var lineIndexAlreadySet:Boolean = false;

		public function CallControlParser()
		{
		}
		
		public static function parse(str:String) : CallControlEvent
		{
			var telephonyEvent:CallControlEvent = null;
			
			try
			{
				XML.ignoreProcessingInstructions=true;
				
				var msg = new XML(str);
			
				OPTrace.debug("msg.localName() = " + msg.localName());
			
				if(msg.localName() == "OF")
				{
					var accountArray:Array = new Array();
					
					var providerList:XMLList = msg.descendants("provider");			
					
					for(var i:Number=0; i < providerList.length(); i++)
					{
						if(providerList[i].child("Enabled").text() == "1")
						{
							var accountList:XMLList = providerList[i].descendants("Account");			
							
							for(var j:Number=0; j < accountList.length(); j++)
							{
								if(accountList[j].child("Enabled").text() == "1") {
	
									var account:Account = new Account();
									
									account.type = msg.@type;
									account.provider = providerList[i].child("Name").text();
									account.user = accountList[j].child("DisplayName").text();
									account.numberOfLines = accountList[j].child("NumberOfCallsPerLines").text();
									accountArray.push(account);
								}
							}
						}
					}
					
					telephonyEvent = new ConfigurationEvent(accountArray);
				}
				else if(msg.localName() == "registration")
				{
					if(msg.@online == "true")
						telephonyEvent = new RegistrationEvent(true, 0);
					else
						telephonyEvent = new RegistrationEvent(false, 0);
						
					telephonyEvent.setProvider(msg.@provider);
					telephonyEvent.setAccount(msg.@account);
					telephonyEvent.setLineIndex(0);
				}
				else if(msg.localName() == "callcontrolerror")			
				{
					OPTrace.debug("callcontrolerror = " + msg.@error);
					telephonyEvent = new CallControlErrorEvent(Number(msg.@error));
				}
				else if(msg.localName() == "dialplanerror")			
				{
					telephonyEvent = new DialPlanErrorEvent(msg.@error);
				}
				else if(msg.localName() == "networkerror")			
				{
					telephonyEvent = new NetworkErrorEvent(msg.@error);
				}
				else if(msg.localName() == "configerror")			
				{
					telephonyEvent = new CallControlErrorEvent(msg.@error);
				}
				else if(msg.localName() == "informational")			
				{
					telephonyEvent = new InformationalEvent(msg.@msg);
				}
				else if(msg.localName() == "frameclient")
				{
					iAmFrameClient = true;

					if (msg.@event == "INCOMING_CANCEL" || msg.@event == "OUTGOING_END" || msg.@event == "INCOMING_END")
					{
						telephonyEvent = new DisconnectedEvent("1", "0");
					}
					else if (msg.@event == "INCOMING_START")
					{
						telephonyEvent = new ConnectedEvent();
					}
					else if (msg.@event == "OUTGOING_START")
					{
						telephonyEvent = new ProceedingEvent("", "");
					}
					else if (msg.@event == "HOOK_OFF")
					{
						telephonyEvent = new HookEvent(false, "gui");
					}
					else if (msg.@event == "HOOK_ON")
					{
						telephonyEvent = new HookEvent(true, "gui");
					}
					else
					{
						telephonyEvent = new FrameClientEvent(msg.@event, msg.@level);
					}
				}
				else if(msg.localName() == "frameclientcallerid")
				{
					iAmFrameClient = true;

					if (msg.@event == "WAITING") {
						telephonyEvent = new IncomingCallEvent(msg.@name, msg.@number, "");
						telephonyEvent.setLineIndex(2);
						lineIndexAlreadySet = true;
					}
					else
						telephonyEvent = new DectCallerIDEvent(msg.@number, msg.@name, "");
				}
				else 
				{
					if(msg.name() == "incomingcall")
					{
						trace("Incoming Call. name=" + msg.@callerName + ", tel=" + msg.@callerNumber + ", time=" + msg.@time);
						telephonyEvent = new IncomingCallEvent(msg.@callerName, msg.@callerNumber, msg.@time);
					}
					else if(msg.name() == "proceeding")
					{
						trace("Call Proceeding. collect=" + msg.@collect + ", dialed=" + msg.@dialed);
						telephonyEvent = new ProceedingEvent(msg.@collect, msg.@dialed);
					}
					else if(msg.name() == "onhook")
					{
						telephonyEvent = new HookEvent(true, msg.@terminal);
					}
					else if(msg.name() == "offhook")
					{
						telephonyEvent = new HookEvent(false, msg.@terminal);
					}
					else if(msg.name() == "remotealerting")
					{
						telephonyEvent = new RemoteAlertingEvent();
					}
					else if(msg.name() == "connected")
					{
						telephonyEvent = new ConnectedEvent();
					}
					else if(msg.name() == "disconnected")
					{
						telephonyEvent = new DisconnectedEvent(msg.@line, msg.@reason);
					}
					else if(msg.name() == "hookflashlong")
					{
						telephonyEvent = new HookFlashLongEvent();
					}
					else if(msg.name() == "hookflash")
					{
						telephonyEvent = new HookFlashEvent();
					}
					else if(msg.name() == "error")
					{
						telephonyEvent = new CallControlErrorEvent(msg.@str);
					}
					else if(msg.name() == "message")
					{
						trace("Message = " + msg.@msg);
						telephonyEvent = new MessageEvent(msg.@msg);
					}
					else if(msg.name() == "voicemail")
					{
						trace("VoiceMail = " + msg.@oldMsgs + "," + msg.@newMsgs + "," + msg.@oldUrgentMsgs + "," + msg.@newUrgentMsgs);
						telephonyEvent = new VoiceMailEvent(msg.@oldMsgs, msg.@newMsgs, msg.@uOldMsgs, msg.@uNewMsgs);
					}
					else if(msg.name() == "dectreginfo")
					{
						telephonyEvent = new DectRegInfoEvent(msg.@status);
					}
					else if(msg.name() == "decthandsetreginfo")
					{
						telephonyEvent = new DectHandsetRegInfoEvent(msg.@handsetid, msg.@state, msg.@handsetname);
					}
					else if(msg.name() == "dectgetreghandsetsinfo")
					{
						var arr:Array = new Array();
						
						//make array of handset id and ipei

						for (var ix:uint = 0; ix < msg.child("*").length(); ix++) {
							var itm:Object = new Object();
							itm.id = msg.handset[ix].@id;
							itm.ipei = msg.handset[ix].@ipei;
							arr.push(itm);
						}

						telephonyEvent = new DectGetRegHandsetsInfoEvent(arr);
					}
					else if(msg.name() == "dectptstatus")
					{
						telephonyEvent = new DectPTStatusEvent(msg.@handsetid, msg.@cradled, msg.@charging, msg.@battery, msg.@software_version, msg.@ bootloader_version, msg.@hardware_model, msg.@handset_name, msg.@ipei);
					}
					else if(msg.name() == "dectintercominfo")
					{
						telephonyEvent = new DectIntercomInfoEvent(msg.@handset1, msg.@handset1_name, msg.@handset2, msg.@handset2_name, msg.@intercom_status);
					}
					else if(msg.name() == "dectrflock")
					{
						telephonyEvent = new DectRFLockEvent(msg.@handsetid);
					}
					else if (msg.name() == "volume")
					{
						telephonyEvent = new VolumeEvent(msg.@level);
					}
					else if(msg.name() == "audioenginestatus")
					{
						telephonyEvent = new AudioEngineStatusEvent(msg.@offhook, msg.@intercom, msg.@volume, msg.@mute, msg.@hwtype);
					}
					else if(msg.name() == "dectcallerid")
					{
						telephonyEvent = new DectCallerIDEvent(msg.@caller_number, msg.@caller_name, msg.@time);
					}
					else if(msg.name() == "ignorecall")
					{
						telephonyEvent = new IgnoreCallEvent();
					}
					else if(msg.name() == "reminder")
					{
						telephonyEvent = new ReminderEvent(msg.@callerName, msg.@callerNumber);
					}
					else if(msg.name() == "dectresethandsetaddressbookresponse")
					{
						telephonyEvent = new DectResetHSABEvent(msg.@handsetid, msg.@error, msg.@error_code);
					}
					else if(msg.name() == "dectsethandsetaddressbookentryresponse")
					{
						telephonyEvent = new DectSetHSABEntryEvent(msg.@handsetid, msg.@error, msg.@error_code);
					}
					else 
						trace("Event not handled.");				
				}

				
				if(telephonyEvent)
				{
					telephonyEvent.setProvider(msg.@provider);
					telephonyEvent.setAccount(msg.@account);
					if (!iAmFrameClient)
						telephonyEvent.setLineIndex(msg.@line);
					else {
						if ( !lineIndexAlreadySet )
							telephonyEvent.setLineIndex(1);
					}
				}
			}
			catch(error:Error)
			{
				trace("Error parsing XML msg.");
			}
			
			return telephonyEvent;
		}
	}
}