﻿package op.extensions.phone
{
	import flash.events.*;
	import flash.utils.Timer;
	import flash.net.*;
	import op.extensions.phone.events.*;


	public class CallControlSocket extends EventDispatcher implements ICallControl
	{
        private var socket:XMLSocket = null;
		private var delay:uint = 5000;
		private var repeat:uint = 0;
		private var reconnectTimer:Timer = null;
		private var serverAddress:String = "10.1.16.10";
		private var serverProtocolPort:uint = 5000;
		private var serverCfgPort:uint = 8081;
		private var configuration:XML = null;
		
		function CallControlSocket()
		{
			var request:URLRequest = new URLRequest("phone.xml");

			trace("CallControlSocket entered.");

			reconnectTimer = new Timer(delay, repeat);

			var loader:URLLoader = new URLLoader();
			
			try
			{
				loader.load(request);
			}
			catch(error:SecurityError)
			{
				trace("A security error has occurred.");
			}
			
			loader.addEventListener(IOErrorEvent.IO_ERROR, errorHandler);
            loader.addEventListener(Event.COMPLETE, loaderCompleteHandlerLocation);

			trace("CallControlSocket exited.");
		}
		
		private function errorHandler(e:IOErrorEvent):void 
		{
			trace("Could not obtain the XML file.");
        }

		
		private function loaderCompleteHandlerLocation(event:Event):void 
		{
			trace("loaderCompleteHandlerLocation entered.");

			try {
				
				var location:XML = new XML(event.target.data);

				serverAddress = location.@address;
				serverProtocolPort = location.@protocol;
				serverCfgPort = location.@cfg;
				
				trace("server address " + serverAddress);
				trace("server port " + serverProtocolPort);
				trace("server cfg port" + serverCfgPort);
				
				initializeSocket();

				// Socket reconnection mechanism
				reconnectTimer.addEventListener(TimerEvent.TIMER, reconnect);

				socket.connect(serverAddress, serverProtocolPort);
			}
			catch (e:TypeError) 
			{
				trace("Could not parse the XML file.");
			}

			trace("loaderCompleteHandlerLocation exited.");
		}
		
		private function reconnect(event:TimerEvent)
		{
			trace("Attempting to reconnect to phone.");
			socket.connect(serverAddress, serverProtocolPort);
		}
		
		/*
		 * Retrieval of the phone configuration so that the GUI
		 * can be rendered for the configured accounts.
		 */
		public function initialize()
		{
			var request:URLRequest = new URLRequest("http://"+serverAddress+":"+serverCfgPort+"/provision/api_get_configuration.php");
			
			trace("Attempting to retrieve configuration.");
			
			var loader:URLLoader = new URLLoader();
			
			try
			{
				loader.load(request);
			}
			catch(error:SecurityError)
			{
				trace("A security error has occurred.");
			}
			
			loader.addEventListener(IOErrorEvent.IO_ERROR, errorHandler);
            loader.addEventListener(Event.COMPLETE, loaderCompleteHandler);
		}
				
		/*
		 * Obtained the configuration information
		 */
		private function loaderCompleteHandler(event:Event):void 
		{
			trace("Successfully retrieved configuration.");
			try 
			{				
				var event2:CallControlEvent = CallControlParser.parse(event.target.data);
				dispatchEvent(event2);
			}
			catch(e:Error)
			{
				trace("Error parsing event message.");
			}
			
		}
		
		public function setDefaultLine(line:CallControlLine)
		{
			var cmd:String = new String();
			cmd = "<default provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\"/>";
			socket.send(cmd);
		}
		
		public function offhook(line:CallControlLine)
		{
			var cmd:String = new String();
			cmd = "<offhook provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\"/>";
			socket.send(cmd);
		}
		
		public function onhook(line:CallControlLine)
		{
			var cmd:String = new String();
			cmd = "<onhook provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\"/>";
			socket.send(cmd);
		}

		public function digit(line:CallControlLine, digit:String)
		{
			var cmd:String = new String();
			cmd = "<digit provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\" digit=\"" + digit +"\"/>";
			socket.send(cmd);
		}

		public function dial(line:CallControlLine, digits:String)
		{
			var cmd:String = new String();
			cmd = "<dial provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\" digits=\"" + digits + "\"/>";
			socket.send(cmd);
		}

		public function redial(line:CallControlLine)
		{
			var cmd:String = new String();
			cmd = "<redial provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\"/>";
			socket.send(cmd);
		}

		public function click2dial(digits:String)
		{
			var cmd:String = new String();
			cmd = "<click2dial digits=\"" + digits + "\"/>";
			socket.send(cmd);
		}

		public function ignore(line:CallControlLine)
		{
			var cmd:String = new String();
			cmd = "<ignorecall provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\"/>";
			socket.send(cmd);
		}
			
		public function hold(line:CallControlLine)
		{
			var cmd:String = new String();
			cmd = "<hold provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\"/>";
			socket.send(cmd);
		}
		
		public function resume(line:CallControlLine)
		{
			var cmd:String = new String();
			cmd = "<resume provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\"/>";
			socket.send(cmd);
		}

		public function atransfer(line:CallControlLine, targetLine:Number)
		{
			var cmd:String = new String();
			cmd = "<atransfer provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\" target=\"" + targetLine + "\"/>";
			socket.send(cmd);
		}

		public function utransfer(line:CallControlLine, teleNumber:String)
		{
			var cmd:String = new String();
			cmd = "<utransfer provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\" target=\"" + teleNumber + "\"/>";
			socket.send(cmd);
		}
		
		public function conference(line1:CallControlLine, line2:CallControlLine)
		{
			var cmd:String = new String();
			line1.getProvider(), line1.getAccount(), line1.getLineIndex(), line2.getProvider(), line2.getAccount(), line2.getLineIndex()
			cmd = "<conference provider=\"" + line1.getProvider() + "\" account=\"" + line1.getAccount() + "\" line=\"" + line1.getLineIndex() + "\" provider2=\"" + line2.getProvider() + "\" account2=\"" + line2.getAccount() + "\" line2=\"" + line2.getLineIndex() + "\"/>";
			socket.send(cmd);
		}
		
		public function message(line:CallControlLine, msg:String)
		{
			var cmd:String = new String();
			cmd = "<message provider=\"" + line.getProvider() + "\" account=\"" + line.getAccount() + "\" line=\"" + line.getLineIndex() + "\" msg=\"" + msg + "\"/>";
			socket.send(cmd);
		}

		public function volume(level:Number)
		{
			var cmd:String = new String();
			cmd = "<volume level=\""+ level + "\"/>";
			socket.send(cmd);
		}

		public function mute(select:String)
		{
			var cmd:String = new String();
			cmd = "<mute/>";
			socket.send(cmd);
		}
		
		public function pageHandsets()
		{
			var cmd:String = new String();
			cmd = "<pageHandsets/>";
			socket.send(cmd);
		}
		
		public function pageCancel()
		{
			var cmd:String = new String();
			cmd = "<pageCancel/>";
			socket.send(cmd);
		}
		
		public function startRegistration()
		{
			var cmd:String = new String();
			cmd = "<startRegistration/>";
			socket.send(cmd);
		}
		
		public function deleteHandset(id:String)
		{
			var cmd:String = new String();
			cmd = "<deleteHandset id=\"" + id + "\"/>";
			socket.send(cmd);
		}
		
		public function getHandsetExtendedStatus(id:String) //pt_status
		{
		}
		
		public function getRegisteredHandsets()
		{
		}
		
		public function intercom(type:String, id:String) //0 = disconnect, 1=alert, 2=connect, 3=busy
		{
		}
		
		public function setHandsetDateTime(id:String, day:String, month:String, mday:String, year:String, hour:String, min:String)
		{
		}
		
		public function getAudioEngineStatus()
		{
		}
		
		public function destructor():void
		{
		}
		
		/*
		 * Registers listeners for various protocol events.
		 */
        private function initializeSocket():void 
		{
			socket = new XMLSocket;
			
            socket.addEventListener(Event.CLOSE, closeHandler);
            socket.addEventListener(Event.CONNECT, connectHandler);
            socket.addEventListener(DataEvent.DATA, onCallControlEvent);
            socket.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
            socket.addEventListener(ProgressEvent.PROGRESS, progressHandler);
            socket.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
        }

        private function closeHandler(event:Event):void 
		{
            trace("closeHandler: " + event);
			dispatchEvent(new RegistrationEvent(false, 100));
			reconnectTimer.start();
        }

        private function connectHandler(event:Event):void 
		{
            trace("connectHandler: " + event);
			if(socket.connected)
			{
				reconnectTimer.stop();
			}
		}

		/*
		 * An event/msg arrived from phone server 
		 */
        private function onCallControlEvent(event:DataEvent):void 
		{
			trace("onCallControlEvent received " + event.data);
		
			try 
			{				
				var event2:CallControlEvent = CallControlParser.parse(event.target.data);
				dispatchEvent(event2);
			}
			catch(e:Error)
			{
				trace("Error parsing event message.");
			}
		}

        private function ioErrorHandler(event:IOErrorEvent):void 
		{
            trace("ioErrorHandler: " + event);
			dispatchEvent(new RegistrationEvent(false, 100));
			reconnectTimer.start();
        }

        private function progressHandler(event:ProgressEvent):void 
		{
            trace("progressHandler loaded:" + event.bytesLoaded + " total: " + event.bytesTotal);
        }

        private function securityErrorHandler(event:SecurityErrorEvent):void 
		{
            trace("securityErrorHandler: " + event);
        }		
	}
}