﻿package op.extensions.phone
{
	import op.external.OPTrace;
	import op.extensions.phone.events.*;
	
	public class FeatureManager
	{
		private var callControl:* = null;
		private var m_lines:Array = null;
		private var numLines:Number = 0;
		private var confcall:Boolean = false;
		private var cr:Boolean = false;
		private var cw:Boolean = false;
		private var domLineIndex:Number = 0;
		private var errorOnLineIndex:int = -1;
		private var errorEvent:* = null;
		private var confcallFlashSwapped:Boolean = false;

		public function FeatureManager(c:*, lines:Array) {
			OPTrace.debug("FeatureManager::Constructor");
			callControl = c;
			
			m_lines = lines;
			
			numLines = m_lines.length;
			
		}
		
		private function resetDomLine() {
			callControl.activeLine = m_lines[0];
			confcall = false;
			cw = false;
			cr = false;
			domLineIndex = 0;
			m_lines[0].reset();
			m_lines[1].reset();
			m_lines[0].dispatchEvent(new LineEvent(LineEvent.EVENT_INVALIDATE_OBJECT));
		}
		
		private function get activeLine () : * {
			return callControl.activeLine;
		}
		
		//public function get inConference() : Boolean { return confcall; }
		//public function get inCW() : Boolean { return cw; }
		
		public function updateState(event:*) : void {
			OPTrace.debug("FeatureManager::updateState");
			var update:Boolean = true;
			
			if ( numLines < 2 ) {
				if ( event is DectCallerIDEvent ) {
					if ( !m_lines[0].offHookCounter )
					{ //singular incoming call
						m_lines[0].loadContacts(true); //true also starts ringback
					}
					else update = false; //do not send dectcaller id event unless singular incoming call
					m_lines[0].setTargetName(event.getName());
					m_lines[0].setTargetNumber(event.getNumber());
				}

				if ( update )
					m_lines[0].updateState(event);
				return;
			}

			//If not a hookflash event and both lines on hook, reset dom line
			if ( !(event is HookFlashEvent) && m_lines[0].getState() == Line.STATE_ON_HOOK && m_lines[1].getState() == Line.STATE_ON_HOOK ) {
				OPTrace.debug("FM::reset dominant line");
				resetDomLine();
			}

			var lineIndex:Number = event.getLineIndex() - 1;
			var olineIndex:Number = lineIndex ? 0 : 1;
			var active_olineIndex:Number = activeLine.getLineIndex() == 1 ? 1 : 0;
			var odomLineIndex:Number = domLineIndex ? 0 : 1;
			
			OPTrace.debug("FeatureManager::lineIndex::" + lineIndex);
			OPTrace.debug("FeatureManager::olineIndex::" + olineIndex);
			OPTrace.debug("FeatureManager::domlineIndex::" + domLineIndex);
			OPTrace.debug("FeatureManager::activeIndex::" + activeLine.getLineIndex());
			OPTrace.debug("FeatureManager::confcall::" + confcall);
			OPTrace.debug("FeatureManager::cw::" + cw );
			OPTrace.debug("FeatureManager::cr::" + cr );

			// ALWAYS CHECK FOR THE CASE WHERE HANDSET IS IN CALL!!!
			if ( event is DectCallerIDEvent ) {
				m_lines[lineIndex].setTargetName(event.getName());
				m_lines[lineIndex].setTargetNumber(event.getNumber());
				if ( !m_lines[olineIndex].offHookCounter && !m_lines[lineIndex].offHookCounter )
				{ //singular incoming call with non call waiting scenario
					//Because the frameclient piggy-backs off of the dect base events... Don't want to start ringing
					//  twice, because we will get two DectCallerIDEvents if we are a frameclient.  We know it is the
					//  second event if there is info in the name/number
					if ( !callControl.isFrameClient ||
						 (callControl.isFrameClient &&
						  (event.getName().length > 0 || event.getNumber().length > 0 ) ) )
						m_lines[lineIndex].loadContacts(true); //true also starts ringback
				}
				else update = false; //do not send dectcaller id event unless singular incoming call
			}
			else if ( event is DectIntercomInfoEvent ) {
				if (	event.getStatus() == "alerting" &&
						m_lines[lineIndex].inBaseIntercomAlert &&
						event.getHS2() == "speakerphone" ) {
							if ( !m_lines[olineIndex].offHookCounter && !m_lines[lineIndex].offHookCounter )
							{ //singular incoming call with non call waiting scenario
								m_lines[lineIndex].loadContacts(true); //true also starts ringback
							}
				}
				else if ( event.getStatus() == "disconnected" ) {
					m_lines[lineIndex].stopRinging();
				}
				update = false; //do not send dectintercom events
			}
			else if ( event is ReminderEvent &&  m_lines[lineIndex].guiOffHook ) {
				m_lines[lineIndex].loadContacts(true); //true also starts ringback
			}
			else if ( event is IncomingCallEvent ) {
				m_lines[lineIndex].setTargetName(event.getCallerName());
				m_lines[lineIndex].setTargetNumber(event.getCallerNumber());
				if ( m_lines[olineIndex].getState() != Line.STATE_ON_HOOK ) {
					cw = true;
				}
				else {
					cw = false;
					update = false; //do not send incoming call event unless call waiting
				}
			}
			else if ( event is ConnectedEvent ) {
				//Need to be careful with this because a hook flash on the hs to answer a call waiting call
				//  will come here as a connected event followed by a hook flash event
				if ( m_lines[olineIndex].getState() == Line.STATE_CONNECTED && //other line to dominant is outgoing; therefore confcall
					 !m_lines[lineIndex].isIncoming ) {
					confcall = true;
					cw = false;
					cr = false;
				}
				else if ( m_lines[olineIndex].getState() == Line.STATE_HOLD ) {
					cr = true;
					cw = false;
					confcall = false;
				}
				else {
					cr = false;
					cw = false;
					confcall = false;
				}
				
				if ( m_lines[olineIndex].getState() == Line.STATE_ON_HOOK )
					domLineIndex = lineIndex;
			}
			else if ( event is DisconnectedEvent ) {
				//This may be due to a hook flash so keep the current state if we need it later
				m_lines[lineIndex].prevOffHookCnt = m_lines[lineIndex].offHookCounter;
				m_lines[lineIndex].prevGUIOffHook = m_lines[lineIndex].guiOffHook;
				
				if ( !(m_lines[lineIndex].inBaseIntercom) ) {
					m_lines[lineIndex].setTargetName("");
					m_lines[lineIndex].setTargetNumber("");
				}
				if ( confcall ) {
					if ( callControl.activeLine.getLineIndex()-1 == lineIndex ) {
						OPTrace.debug("FM::DIS::swap line");
						callControl.swap_line();
						confcallFlashSwapped = true;
					}
				}
				if ( m_lines[olineIndex].getState() == Line.STATE_HOLD )
					cr = true;
				else
					cr = false;
				cw = false;
				confcall = false;
			}
			else if ( event is IgnoreCallEvent ) {
				cw = false;
			}
			else if ( event is HookFlashEvent ) {
				var updateOLineState:Boolean = false;
				OPTrace.debug("FeatureManager::activeIndex::" + activeLine.getLineIndex());
				OPTrace.debug("FeatureManager::lineIndex::" + event.getLineIndex());
				//If this line is active send the event to the other line if it is not connected (HS cw answer scenario)
				if ( activeLine.getLineIndex() == event.getLineIndex() && m_lines[olineIndex].getState() != Line.STATE_CONNECTED )
					updateOLineState = true;

				if ( (	m_lines[lineIndex].getState() == Line.STATE_CONNECTED ||
						m_lines[lineIndex].getState() == Line.STATE_REMOTE_ALERTING ) &&
						m_lines[olineIndex].getState() == Line.STATE_HOLD && //other line to dominant is outgoing; therefore confcall
						!m_lines[odomLineIndex].isIncoming ) {
					OPTrace.debug("OTHER LINE WAS OUTGOING");
					confcall = true;
					update = false;
				}
				else {
					if (!confcallFlashSwapped)
						callControl.swap_line();
					confcallFlashSwapped = false;
					
					//If this line is anything but on hook, process the flash event
					if (m_lines[lineIndex].getState() == Line.STATE_ON_HOOK)
						update = false;
				}
				
				if (updateOLineState)
					m_lines[olineIndex].updateState(event);

				//This should fix the issue with multiple terminals off hook when hook flash happens
				if (m_lines[lineIndex].prevOffHookCnt > 0) {
					m_lines[olineIndex].offHookCounter = m_lines[lineIndex].prevOffHookCnt;
					m_lines[olineIndex].guiOffHook = m_lines[lineIndex].prevGUIOffHook;
				}
                else {
                    m_lines[olineIndex].offHookCounter = m_lines[lineIndex].offHookCounter;
					m_lines[olineIndex].guiOffHook = m_lines[lineIndex].guiOffHook;
                }
			}
			else if ( confcall && event is HookEvent ) {
				OPTrace.debug("CONF HOOK");
				m_lines[olineIndex].updateState(event);
			}

			// if an error occurred, keep the lineIndex for the error and the actual event
			// need this during remote alerting as well bug #5780
			if ( (	event is CallControlErrorEvent ||
					event is RemoteAlertingEvent ) &&
					m_lines[lineIndex].getState() != Line.STATE_ON_HOOK ) {
				errorOnLineIndex = lineIndex;
				errorEvent = event;

				// if this is a "Not Implemented" error and we were trying to establish a conf call, fix the state
				if ( confcall && event.getErrorMessage() == "Not Implemented" )
					confcall = false;
			}
			else {
				// no error occurred
				errorOnLineIndex = -1;
				errorEvent = null;
			}

			OPTrace.debug("FeatureManager::confcall_end::" + confcall);
			OPTrace.debug("FeatureManager::cw_end::" + cw );
			OPTrace.debug("FeatureManager::cr_end::" + cr );

			if ( update )
				m_lines[lineIndex].updateState(event);

			// if line is already on hook and receiving disconnected event, do not set CR
			if ( m_lines[lineIndex].getState() == Line.STATE_ON_HOOK &&
				 m_lines[olineIndex].getState() != Line.STATE_ON_HOOK ) {
				if (m_lines[olineIndex].getState() != Line.STATE_OFF_HOOK)
					cr = true;
				domLineIndex = olineIndex;
			}

			// if not conf and line is remote alerting and otherline is on hold, change the flash button to join
			if ( 	!confcall && 
					m_lines[lineIndex].getState()  == Line.STATE_REMOTE_ALERTING &&
					m_lines[olineIndex].getState() == Line.STATE_HOLD &&
					m_lines[lineIndex].guiOffHook ) {
				m_lines[lineIndex].setButtonsOffHookJoin();
				m_lines[lineIndex].dispatchEvent(new LineEvent(LineEvent.EVENT_INVALIDATE_OBJECT));					 
			}
			else if (	cr &&
					m_lines[lineIndex].getState() == Line.STATE_ON_HOOK &&
					m_lines[olineIndex].offHookCounter ) {
				m_lines[lineIndex].setButtonsOnHookFlash();
				m_lines[lineIndex].dispatchEvent(new LineEvent(LineEvent.EVENT_INVALIDATE_OBJECT));
			}
			else if (	event is HookFlashEvent &&
					activeLine.getState() != Line.STATE_ON_HOOK &&
					activeLine.guiOffHook ) {
                activeLine.setButtonsOffHook();
				activeLine.dispatchEvent(new LineEvent(LineEvent.EVENT_INVALIDATE_OBJECT));
			}

			//Make sure mute is off
			//If there are no GUI lines off hook and
			//  not in intercom or
			//  in intercom and this is an intercom event to disconnect the speakerphone
			if (!m_lines[lineIndex].guiOffHook && !m_lines[olineIndex].guiOffHook &&
				(!(m_lines[lineIndex].inBaseIntercom) ||
				  (m_lines[lineIndex].inBaseIntercom && event is DectIntercomInfoEvent && event.getStatus() == "disconnected" && event.getHS2() == "speakerphone" ) ) ) {
				if ( m_lines[lineIndex].getMuteState() )	m_lines[lineIndex].toggleMute();
				if ( m_lines[olineIndex].getMuteState() )	m_lines[olineIndex].toggleMute();
			}
		}
		
		public function onSoftKeyEvent(event:*) : void {
			// NO LINE INDEX, only based on active Line
			OPTrace.debug("FeatureManager::onSoftKeyEvent");
			
			if ( numLines < 2 ) {
				m_lines[0].onSoftKeyEvent(event);
				return;
			}
			
			var lineIndex:Number = activeLine.getLineIndex() - 1;
			var olineIndex:Number = lineIndex ? 0 : 1;
			var odomLineIndex:Number = domLineIndex ? 0 : 1;
			
			OPTrace.debug("FeatureManager::lineIndex::" + lineIndex);
			OPTrace.debug("FeatureManager::olineIndex::" + olineIndex);
			OPTrace.debug("FeatureManager::odomlineIndex::" + odomLineIndex);
			OPTrace.debug("FeatureManager::confcall::" + confcall);
			OPTrace.debug("FeatureManager::cw::" + cw );
			OPTrace.debug("FeatureManager::cr::" + cr );

			if (event.m_eventId == LineEvent.EVENT_HOOK_FLASH) {
				if ( confcall ) {
					//Base flash during a conf. call. Need to swap lines and hookflash on other dom line.
					callControl.swap_line();
					callControl.hookFlash(m_lines[odomLineIndex].getCallControlLine(), event.m_duration);
					m_lines[odomLineIndex].onSoftKeyEvent(new LineEvent(LineEvent.EVENT_END_CALL));
                    
					//Additionally, need to make sure gui off hook state is true on dom line.  This is needed
					//  if the base joined a call that was previously setup by HS.
					if (!m_lines[domLineIndex].guiOffHook)
						m_lines[domLineIndex].onSoftKeyEvent(new LineEvent(LineEvent.EVENT_ANSWER));

					confcall = false;
					cr = false;
					cw = false;
				}
				else if ( (	m_lines[lineIndex].getState() == Line.STATE_CONNECTED ||
							m_lines[lineIndex].getState() == Line.STATE_REMOTE_ALERTING ) &&
							m_lines[olineIndex].getState() == Line.STATE_HOLD  && //other line to dominant is outgoing; therefore confcall
							!m_lines[odomLineIndex].isIncoming ) {
					OPTrace.debug("OTHER LINE WAS OUTGOING");
					confcall = true;
					cr = false;
					cw = false;
					callControl.hookFlash(m_lines[lineIndex].getCallControlLine(), event.m_duration);
					m_lines[olineIndex].onSoftKeyEvent(event);
					// do not send hookflash event to current activeline to change
					
					// change the Join button back to Flash
					m_lines[lineIndex].setButtonsOffHook();
					m_lines[lineIndex].dispatchEvent(new LineEvent(LineEvent.EVENT_INVALIDATE_OBJECT));
				}
				else if ( m_lines[lineIndex].getState() == Line.STATE_OFF_HOOK ||
						  m_lines[lineIndex].getState() == Line.STATE_PROCEEDING ||
						  m_lines[lineIndex].getState() == Line.STATE_REMOTE_ALERTING ||
						  m_lines[lineIndex].getState() == Line.STATE_CONNECTED ) {
					OPTrace.debug("FM:OFFHOOK OR CONNECTED::SWAP");
					callControl.swap_line();
					m_lines[lineIndex].onSoftKeyEvent(event);
					m_lines[olineIndex].onSoftKeyEvent(event);
				}
				else if ( cw ) {
					//line was already swapped at incoming call (this is an Answer Call)
					OPTrace.debug("FM::CW");
					if( m_lines[lineIndex].getState() == Line.STATE_ON_HOOK ) //during incoming call event
						m_lines[lineIndex].onSoftKeyEvent(new LineEvent(LineEvent.EVENT_ANSWER));
					else
						m_lines[lineIndex].onSoftKeyEvent(event);
					m_lines[olineIndex].onSoftKeyEvent(event);

					//make sure the off hook counters match
					while (m_lines[lineIndex].offHookCounter < m_lines[olineIndex].offHookCounter) {
						m_lines[lineIndex].updateState(new HookEvent(false, "gui"));
					}
				}
				else if ( cr ) {
					OPTrace.debug("FM::CR");
					callControl.hookFlash(m_lines[olineIndex].getCallControlLine(), event.m_duration);
					callControl.swap_line(); //this is used for flash button, 
											 //otherwise CR popup should switch the line and this will switch to the correct line
					m_lines[olineIndex].onSoftKeyEvent(event);
				}
				else {// send hookflash to current line
					OPTrace.debug("FM::ALL ELSE");
					m_lines[lineIndex].onSoftKeyEvent(event);
				}

				//This should fix the issue with multiple terminals off hook when hook flash happens
				if ( m_lines[lineIndex].getState() != Line.STATE_ON_HOOK ) {
					m_lines[olineIndex].offHookCounter = m_lines[lineIndex].offHookCounter;
					m_lines[olineIndex].guiOffHook = m_lines[lineIndex].guiOffHook;
				}
			}
			else if (event.m_eventId == LineEvent.EVENT_END_CALL) {
				if ( confcall ) {
					OPTrace.debug("FeatureManager::END CALL ON CONF CALL");
					if (m_lines[lineIndex].offHookCounter == 1) 
						resetDomLine();
					else {
						m_lines[lineIndex].onSoftKeyEvent(event);
						m_lines[olineIndex].onSoftKeyEvent(event);
					}
				}
				else {
					m_lines[lineIndex].onSoftKeyEvent(event);
					
					if ( errorOnLineIndex == lineIndex ) {
						// this line previously failed, so no disconnect event is coming
						// we need to update the state
						updateState(errorEvent);
					}
                    
					//Because end will put us on hook (and enable the intercom btn), we need to disable it when the other line is in use
					if ( m_lines[olineIndex].getState() != Line.STATE_ON_HOOK ) {
						m_lines[lineIndex].getButtonArray().push( { eventId:LineEvent.EVENT_INTERCOM, enabled:false } );
                        m_lines[olineIndex].setButtonsOnHookJoin();
                        m_lines[olineIndex].getButtonArray().push( { eventId:LineEvent.EVENT_INTERCOM, enabled:false } );
                        //If there is another device off hook on this line, put the other line on-hook as well
                        if ( m_lines[lineIndex].offHookCounter > 0 )
                            m_lines[olineIndex].decOffHook();
                    }
				}
			}
			else if (event.m_eventId == LineEvent.EVENT_NEW_CALL ||
                     event.m_eventId == LineEvent.EVENT_JOIN) {
                m_lines[lineIndex].onSoftKeyEvent(event);
				if (m_lines[olineIndex].getState() != Line.STATE_ON_HOOK)
				{
					// to avoid setting the other line's offHookCounter and guiOffHook when the other line is idle
					m_lines[olineIndex].setButtonsOffHook();
					m_lines[olineIndex].incOffHook();
				}
			}
			else if (event.m_eventId == LineEvent.EVENT_IGNORE) {
				m_lines[lineIndex].onSoftKeyEvent(event);
				if( cw )
					callControl.swap_line();
				if (!callControl.isFrameClient)
					//generate an ignore event and propagate (for call logs mainly)
					callControl.onCallControlEvent("<ignorecall line=\"" + (lineIndex+1) + "\" />");
			}
			else {
				OPTrace.debug("FM::Not a hookflash");
				m_lines[lineIndex].onSoftKeyEvent(event);
			}

			//If there are no lines off hook, make sure mute is off
			if (!m_lines[lineIndex].guiOffHook &&
				!m_lines[olineIndex].guiOffHook) {
				if ( m_lines[lineIndex].getMuteState() )	m_lines[lineIndex].toggleMute();
				if ( m_lines[olineIndex].getMuteState() )	m_lines[olineIndex].toggleMute();
			}
		}
		
	}
	
}
