﻿package op.utils
{
	/**********************************************************
	 * desc: A base class definition for a library MovieClip that
	 * implements a button with two states and a dynamic label with 
	 * multiple color states.  This class assumes the library asset 
	 * has key frames at frame 1 and 2 of the asset’s local timeline 
	 * representing the inactive and active states respectively.  
	 * 
	 * Additionally the asset must contain the following on a layer 
	 * that spans the state key frames:  
	 * 
	 * TextField, id = lbl_txt
	 * 
	 */
	
	import flash.display.MovieClip;
	import flash.display.SimpleButton;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	
	public class OPBtnToggleWTI extends MovieClip
	{
		private var upColor:int = 0xFFFFFF;
		private var downColor:int = 0x000000;
		private var selectedUpColor:int = 0xFFFFFF;
		private var selectedDownColor:int = 0x000000;
		
		public function OPBtnToggleWTI()
		{
			stop();
			var txt:TextField = lbl_txt;
			txt.mouseEnabled = false;
			txt.embedFonts = true;
			
			addEventListener(MouseEvent.MOUSE_DOWN, onDown, false, 0, true);
			addEventListener(MouseEvent.MOUSE_UP, onUp, false, 0, true);
			addEventListener(MouseEvent.MOUSE_OUT, onOut, false, 0, true);
		}
		
		public function setColors( ...args )
		{
			
			var len = args.length;
			
			switch(len)
			{
				case 4:
					upColor = args[0];
					downColor = args[1];
					selectedUpColor = args[2];
					selectedDownColor = args[3];
					break;
				case 3:
					upColor = args[0];
					downColor = args[1];
					selectedUpColor = args[2];
					selectedDownColor = args[2];
					break;
				case 2:
					upColor = args[0];
					downColor = args[1];
					selectedUpColor = args[0];
					selectedDownColor = args[1];
					break;
				case 1:
					upColor = args[0];
					downColor = args[0];
					selectedUpColor = args[0];
					selectedDownColor = args[0];
					break;
			}
			
			onUp();
		}
		
		public function set lbl(str:String)
		{
			lbl_txt.text = str;
		}
		
		public function get lbl():String
		{
			return lbl_txt.text;
		}
		
		public function get isOn():Boolean
		{
			return currentFrame == 2;
		}
		
		public function set isOn(b:Boolean)
		{
			gotoAndStop(b ? 2 : 1);
			onUp();
		}
		
		public function onDown(e:MouseEvent)
		{
			lbl_txt.textColor = isOn ? selectedDownColor : downColor;
		}
		
		public function onOut(e:MouseEvent)
		{
			onUp(e);
		}
		
		public function onUp(e:MouseEvent = null)
		{
			lbl_txt.textColor = isOn ? selectedUpColor : upColor;
		}
		
		public override function set enabled(b:Boolean):void
		{
			alpha = b ? 1 : .25;
			mouseEnabled = b;
			mouseChildren = b;
			super.enabled = b;
		}
		
	}
	
}