﻿package op.utils
{
	/**********************************************************
	 *  desc: A base class definition for a library MovieClip that 
	 * implements a custom cell renderer with an instance of 
	 * OPBtnToggle for a flash List component using the 
	 * ICellRenderer implementation.  This class assumes 
	 * that the following assets are sitting on the stage 
	 * at frame 1 of the asset’s local timeline:
	 *  
	 *  TextField, id=lbl_txt
	 *  SimpleButton, id=btn
	 *  MovieClip, id=selected_mc
	 *  MovieClip linked to OPBtnToggle, id=check_btn
	 *   		
	 */
	
	import flash.display.MovieClip;
    import flash.text.TextField;
    import fl.controls.listClasses.ICellRenderer;
    import fl.controls.listClasses.ListData;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.DropShadowFilter;
	import flash.system.fscommand;
	import op.utils.OPEvent;
	
    public class OPCellCheckBox extends MovieClip implements ICellRenderer 
	{
		public static const CHECKSTATUS:String = "check_status";
		
		private var _listData:ListData;
        private var _data:Object;
        private var _selected:Boolean;
		
		public function OPCellCheckBox()
		{
			super();
			
			lbl_txt.embedFonts = true;
			lbl_txt.mouseEnabled = false;
			
			selected_mc.filters = [new DropShadowFilter(5, 45, 0x000000, .75)];
			selected_mc.visible = false;
			
			check_btn.addEventListener(MouseEvent.CLICK, onChecked, false, 0, true);
		}

		public function set data(d:Object):void 
		{
            _data = d;
			lbl_txt.text = d.Name;
			check_btn.isOn = d.checked;
        }
		
        public function get data():Object 
		{
            return _data;
        }
        
		public function set listData(ld:ListData):void 
		{
            _listData = ld;
        }
        
		public function get listData():ListData 
		{
            return _listData;
        }
		
		public function onChecked(e:MouseEvent)
		{
			check_btn.isOn = !check_btn.isOn;
			_data.checked = check_btn.isOn;
			
			//prevent list onChange Event
			e.stopPropagation();
			
			//dispatch a CHECKSTATUS event.
			dispatchEvent( new Event(OPCellCheckBox.CHECKSTATUS, true, false) );
		}

		public function set selected(s:Boolean):void 
		{
            _selected = s; 
			selected_mc.visible = s;
			lbl_txt.textColor = s ? 0xFFFFFF : 0x000000;
        }
        
		public function get selected():Boolean 
		{
            return _selected;
        }
        
		public function setSize(w:Number, h:Number):void 
		{
			btn.width = w;
			selected_mc.width = w - 15;
        }
        
		public function setStyle(style:String, value:Object):void 
		{
        }
        
		public function setMouseState(state:String):void
		{        
			trace("Mouse " + state);
		}

    }
}    

