﻿package op.utils
{
	/**********************************************************
	 * desc: A base class definition for a library MovieClip that 
	 * implements a custom cell renderer for a flash List component 
	 * using the ICellRenderer implementation.  This class assumes 
	 * that the following assets are sitting on the stage at frame 1 
	 * of the asset’s local timeline:
	 * 
	 * TextField, id = lbl_txt
	 * SimpleButtons, id = btn
	 * MovieClip, id = selected_mc
	 * MovieClip linked to OPBtnToggle, id = check_btn
	 * 
	 * The following styles can be set using the setRendererStyle method 
	 * on the List component instance:
	 * 
	 * displayPath:String – defines the object attribute name that will be used to set lbl_txt, the default value is “lbl”
	 * showSelection:Boolean – set to false to prevent the selected state from rendering, the default value is true
	 * 
	 */
		
	import flash.display.MovieClip;
    import flash.text.TextField;
    import fl.controls.listClasses.ICellRenderer;
    import fl.controls.listClasses.ListData;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.DropShadowFilter;
	import op.utils.OPEvent;
	
    public class OPCellOption extends MovieClip implements ICellRenderer 
	{
		
		private var _listData:ListData;
        private var _data:Object;
        private var _selected:Boolean;
		
		private var displayPath:String = "lbl";
		private var showSelection = true;
		
		public function OPCellOption()
		{
			super();
			
			lbl_txt.embedFonts = true;
			lbl_txt.mouseEnabled = false;
			
			selected_mc.filters = [new DropShadowFilter(5, 45, 0x000000, .75)];
			selected_mc.visible = false;
		}

		public function set data(d:Object):void 
		{
            _data = d;
			lbl_txt.text = d[displayPath]
        }
		
        public function get data():Object 
		{
            return _data;
        }
        
		public function set listData(ld:ListData):void 
		{
            _listData = ld;
        }
        
		public function get listData():ListData 
		{
            return _listData;
        }
		
		public function set selected(s:Boolean):void 
		{
            _selected = s;
			
			if (showSelection)
			{
				selected_mc.visible = s;
				lbl_txt.textColor = s ? 0xFFFFFF : 0x000000;
			}
			
        }
        
		public function get selected():Boolean 
		{
            return _selected;
        }
        
		public function setSize(w:Number, h:Number):void 
		{
			btn.width = w;
			selected_mc.width = w - 15;
        }
        
		public function setStyle(style:String, value:*):void 
		{
			this[style] = value;
        }
        
		public function setMouseState(state:String):void
		{        
			trace("Mouse " + state);
		}

    }
}    

