﻿package op.utils
{
	import fl.controls.ScrollPolicy;
	import fl.controls.TextArea;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.GlowFilter;
	import flash.text.TextFormat;
	import flash.utils.getTimer;
	
	public class OPTextArea extends TextArea
	{
		private var bTrap:Boolean = false;
		private var bSlide:Boolean = false;
		private var lastY:Number;
		private var speedY:Number;
		private var count:uint;
		private var accel:Number = 8;
		private var deccel:Number = 0.98;
		private var startDelta:Number = 3;		
		private var rate:uint;
		private var txtFormat:TextFormat;
				
		public function OPTextArea(tf:* = null)
		{
			super();
			//set background
			var bg:Sprite = new Sprite();
			bg.graphics.beginFill(0xEBEBEB, 1);
			bg.graphics.drawRect(0, 0, 1, 1);
			bg.graphics.endFill();
						
			if (tf == null)
			{
				tf = new TextFormat();
				tf.size = 16;
				tf.font = "Arial";
				tf.color = 0x000000;
				tf.rightMargin = 12;
			}
			//set right margin to 12 so that text won't move when vertical scrollbar appears and disappears
			txtFormat = tf;
			txtFormat.rightMargin = 12;
			
			setStyle("focusRectSkin", new Sprite());
			setStyle("upSkin", bg);
			setStyle("embedFonts", true);
			setStyle("textFormat", txtFormat);			
			setStyle("trackUpSkin",  OPSBTrack );
			setStyle("trackDownSkin", OPSBTrack );
			setStyle("trackDisabledSkin", OPSBTrack );
			setStyle("trackOverSkin", OPSBTrack );
			setStyle("thumbUpSkin",  OPSBThumb );
			setStyle("thumbDownSkin", OPSBThumb );
			setStyle("thumbDisabledSkin", OPSBThumb );
			setStyle("thumbOverSkin", OPSBThumb );
			setStyle("thumbIcon", new Sprite() );
			setStyle("upArrowUpSkin", OPSBArrow);
			setStyle("upArrowOverSkin", OPSBArrow );
			setStyle("upArrowDownSkin", OPSBArrow);
			setStyle("upArrowDisabledSkin", OPSBArrow );
			setStyle("downArrowUpSkin", OPSBArrow);
			setStyle("downArrowOverSkin", OPSBArrow);
			setStyle("downArrowDownSkin", OPSBArrow);
			setStyle("downArrowDisabledSkin", OPSBArrow);
			
			verticalScrollBar.setStyle("scrollBarWidth", 12);
			verticalScrollBar.setStyle("scrollArrowHeight", 5);
			verticalScrollPolicy = ScrollPolicy.OFF;
			editable = false;
			textField.mouseEnabled = false;
			
			filters = [new GlowFilter(0x000000, 0.75, 0, 8 , 2 , 1, true, false) ];
			addEventListener(MouseEvent.MOUSE_DOWN, onDown, false, 0, true);
		}
		
		private function onDown(e:MouseEvent)
		{
			trace("OPTextArea:: " + this.name + " :: onDown");
			mouseChildren = true;
			
			bTrap = false;
			bSlide = false;
			
			lastY = mouseY;
			speedY = 0;
			count = 1;
			deccel = 0.9;
		
			stage.addEventListener(MouseEvent.MOUSE_UP, onUp, false, 0, true);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, onMove, false, 0, true);
		}
		
		private function onFrame(e:Event)
		{
			trace( "OPTextArea::" + this.name + "::onFrame: rate = " + (1000 / (getTimer() - rate) ) );
			rate = getTimer();
			
			trace("OPTextArea::"+this.name+"::onFrame: bSlide = " + bSlide + ", speedY = " + speedY);
			
			if (bSlide)
			{
				verticalScrollPosition -= speedY;
				speedY *= deccel;
				
				if (Math.abs (speedY) < 1)
				{
					bSlide = false;
					mouseChildren = true;
					verticalScrollPolicy = ScrollPolicy.OFF;
					
					//reset right margin to 12 when vertical scrollbar disppears
					var vspos:int = verticalScrollPosition;
					txtFormat.rightMargin = 12;
					setStyle("textFormat", txtFormat);			
					verticalScrollPosition = vspos;

					removeEventListener(Event.ENTER_FRAME, onFrame);
					
				}
							
			} else {
				
				speedY *= 0.25;
				trace("OPTextArea::"+this.name+"::onFrame::reducing, speedY = " + speedY);
			}
			
			if (verticalScrollPosition == 0 || verticalScrollPosition == maxVerticalScrollPosition)
					speedY = 0;
		}
		
		private function onMove(e:MouseEvent)
		{
			trace( "OPTextArea::" + this.name + ":: y: " + this.mouseY + ", vp: " + this.verticalScrollPosition);
			
			var difference = this.mouseY - lastY;
			trace( "OPTextArea::" + this.name + ":: difference: " + difference)
			if (bTrap)
			{
				
				speedY += difference;
				lastY = mouseY;
				count++;
				
				if ( 1 < Math.abs(difference) )
				{
					trace( "OPTextArea::" + this.name + ":: moving, speed: " + speedY)
					verticalScrollPosition -= speedY;
					
				}
			
			} else if (startDelta < Math.abs(difference) ){
			
				bTrap = true;
				mouseChildren = false;
				verticalScrollPolicy = ScrollPolicy.AUTO;
					
				//When it is possible to have vertical scroll bar, reset margin to 0 so that scrollbar won't push the text to left
				if (height < textHeight)
				{
					var vspos:int = verticalScrollPosition;
					txtFormat.rightMargin = 0;
					setStyle("textFormat", txtFormat);			
					verticalScrollPosition = vspos;
				}
				
				addEventListener(Event.ENTER_FRAME, onFrame, false, 0, true);
				rate = getTimer();
			}
		}
		
		private function onUp(e:MouseEvent)
		{
			trace("OPTextArea:: " + this.name + " :: onUp");
			if (!bSlide)
			{
				trace("onUp :: not sliding");
				
				stage.removeEventListener(MouseEvent.MOUSE_MOVE, onMove);
				stage.removeEventListener(MouseEvent.MOUSE_UP, onUp);
			
				bTrap = false;
				
				speedY += mouseY - lastY;
				speedY = (speedY * accel) / count;
			
				if ( speedY )
				{
					bSlide = true;
				} else {
					mouseChildren = true;
					verticalScrollPolicy = ScrollPolicy.OFF;
					
					//reset right margin to 12 when vertical scrollbar disppears
					var vspos:int = verticalScrollPosition;
					txtFormat.rightMargin = 12;
					setStyle("textFormat", txtFormat);			
					verticalScrollPosition = vspos;

					removeEventListener(Event.ENTER_FRAME, onFrame);
				}
			}
		}
	}
}